unit Udump;

{
  IIC-Bus Universal

  Freeware, Copyright 1996, Matthias Weingart;
  comments, bug reports to matthias@penthouse.boerde.de
}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls, Grids, IniFiles, Whexgrd, unii2c,
  Menus;
const
     CEEPROMTimeout=500; {ms timeout until byte written}

type
  TDumpForm = class(TForm)
    WHexGrid1: TWHexGrid;
    DumpPanel: TPanel;
    ReadBtn: TButton;
    TypeGrp: TRadioGroup;
    SizeEdit: TEdit;
    Label1: TLabel;
    AdrEdit: TEdit;
    Label2: TLabel;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Load1: TMenuItem;
    Save1: TMenuItem;
    Exit1: TMenuItem;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    procedure ReadBtnClick(Sender: TObject);
    procedure WHexGrid1GetHexValue(Sender: TObject; AAdr: Longint;
      var Value: Byte);
    procedure WHexGrid1SetHexValue(Sender: TObject; AAdr: Longint;
      const Value: Byte);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Load1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure Save1Click(Sender: TObject);
  private
    { Private-Deklarationen }
    FSize: Integer;
    FAdr:  Integer;
    procedure Read;
    procedure ReadFromNvm( AAdr: Longint; Count: LongInt; var data: array of byte );
    procedure WriteToNvm( AAdr: Longint; Count: LongInt; var data: array of byte );
  public
    { Public-Deklarationen }
    I2C:   TPI2C;
    Ini:   TIniFile;
  end;

var
  DumpForm: TDumpForm;

implementation

{$R *.DFM}

procedure TDumpForm.FormCreate(Sender: TObject);
begin
    FAdr:=$ff;
    Ini := TIniFile.Create( ChangeFileExt(Application.ExeName,'.ini' ));

    SizeEdit.Text:=Ini.ReadString( 'Hexdump', 'Size', '128' );
    AdrEdit.Text:=Ini.ReadString( 'Hexdump', 'Adr', 'A0' );
    TypeGrp.ItemIndex:=Ini.ReadInteger( 'Hexdump', 'Type', 0 );
end;

procedure TDumpForm.FormDestroy(Sender: TObject);
begin
    Ini.WriteString( 'Hexdump', 'Size', SizeEdit.Text );
    Ini.WriteString( 'Hexdump', 'Adr', AdrEdit.Text );
    Ini.WriteInteger( 'Hexdump', 'Type', TypeGrp.ItemIndex );
end;

procedure TDumpForm.Read;
begin
   try
     FSize:=StrToInt( SizeEdit.Text );
     FAdr :=StrToInt( '$0'+AdrEdit.Text );
     WHexGrid1.HexEndAdr:= FSize-1;
     WHexGrid1.Invalidate;
   except
     on EConvertError do Application.Messagebox('Invalid size or address selected.','Error',MB_APPLMODAL);
   end;
end;

procedure TDumpForm.ReadBtnClick(Sender: TObject);
begin
     Read;
end;

procedure TDumpForm.ReadFromNvm( AAdr: Longint; Count: LongInt; var data: array of byte );
begin
  {Screen.Cursor := crHourglass;}
  try
   if FAdr<>$ff then
   begin
     case TypeGrp.ItemIndex of
       0: I2C.ReadNvm( tadr8, FAdr, AAdr , Count, data );
       1: I2C.ReadNvm( tadr16,FAdr, AAdr , Count, data );
     end;
   end;
  finally
    {Screen.Cursor := crDefault;  }
  end;
end;

procedure TDumpForm.WHexGrid1GetHexValue(Sender: TObject; AAdr: Longint;
  var Value: Byte);
var data: array[0..1] of byte;
begin
   Value:=$ff;
   if FAdr<>$ff then
   begin
     ReadFromNvm( AAdr, 1, data );
     Value:=data[0];
   end;
end;

procedure TDumpForm.WriteToNvm( AAdr: Longint; Count: LongInt; var data: array of byte );
var o:LongInt;
begin                            
  {Screen.Cursor := crHourglass;  }
  try
   if FAdr<>$ff then
   begin
     case TypeGrp.ItemIndex of
       0: I2C.WriteNvm( tadr8, FAdr, AAdr, Count, data );
       1: I2C.WriteNvm( tadr16,FAdr, AAdr, Count, data );
     end;
   end;
  finally
    {Screen.Cursor := crDefault;}
  end;
end;

procedure TDumpForm.WHexGrid1SetHexValue(Sender: TObject; AAdr: Longint;
  const Value: Byte);
var data: array[0..1] of byte;
begin
   data[0]:=Value;
   WriteToNvm( AAdr, 1, data );
end;

procedure TDumpForm.Load1Click(Sender: TObject);
var F: file;
    Buf: array[0..1] of Byte;
    Adr: LongInt;
    Num: word;
begin
  if OpenDialog1.Execute then
  begin
    AssignFile(F, OpenDialog1.FileName);
    Reset(F, 1);		{ Satzlnge = 1 }
    Adr:=0;
    repeat
        BlockRead(F, Buf, 1, Num);
        if Num<>0 then
        begin
           WriteToNvm( Adr, 1, Buf );
        end;
        Adr:=Adr+1;
    until (Num = 0) or (Adr>=FSize);
    CloseFile( F );
    Read;
  end;
end;

procedure TDumpForm.Save1Click(Sender: TObject);
var F: file;
    Buf: array[0..1] of Byte;
    Adr: LongInt;
    Num: word;
begin
  FSize:=StrToInt( SizeEdit.Text );
  if SaveDialog1.Execute then
  begin
    AssignFile(F, SaveDialog1.FileName);
    Rewrite(F, 1);		{ Satzlnge = 1 }
    Adr:=0;
    repeat
        ReadFromNvm( Adr, 1, Buf );
        BlockWrite(F, Buf, 1, Num);
        Adr:=Adr+1;
    until Adr>=FSize;
    CloseFile( F );
  end;
end;

procedure TDumpForm.Exit1Click(Sender: TObject);
begin
     Close;
end;

end.


